/*
AnalysIRmodRPi.c
A command line utility for Raspberry Pi Model B, to measure the modulation of IR signals.

Step 1: Save files to your desired directory on RPi. (we picked /etc/wiringPi/sketches/sketch_AnalysIR_RPi)
Step 2: Compile 'AnalysIRmodRPi.c' with 'gcc' as per the command below.
Step 3: Wire up the Raspberry Pi as indicated in the Getting Started Guide.
Step 4: Run the utility as indicated in one of the 2 commands below. Use the higher priority command, unless you have a reson no to.
Step 5: You may need to use sudo befoe the commands below.

Compile as follows:

	gcc  -o AnalysIRmodRPi AnalysIRmodRPi.c -lwiringPi

Run as follows: (may need to use 'sudo' also) - binary output goes to SDOUT/terminal window
	cd /etc/wiringPi/sketches/sketch_AnalysIR_RPi
	./AnalysIRmodRPi


or with even higher priority
	nice -n -20 ./AnalysIRmodRPi

useful command:  route add default gw 192.168.137.1 eth0  //defines gateway for n/w, if missing (change IP address according to your own LAN)

Output of gpio readall shell  command on RPi Model B
+----------+------+------+--------+
|      0   |  17  |  11  | GPIO 0 | setup LIRC IR RX on this pin //can be changed in .conf file of LIRC
|      1   |  18  |  12  | GPIO 1 | IR Modulated signal   - 1    //can be changed in definitions below
|      2   |  27  |  13  | GPIO 2 | IR demodulated signal - 2    //can be changed in definitions below
|      3   |  22  |  15  | GPIO 3 |
|      4   |  23  |  16  | GPIO 4 |
|      5   |  24  |  18  | GPIO 5 |
|      6   |  25  |  22  | GPIO 6 |
|      7   |   4  |   7  | GPIO 7 |
|      8   |   2  |   3  | SDA    | 
|      9   |   3  |   5  | SCL    | 
|     10   |   8  |  24  | CE0    |
|     11   |   7  |  26  | CE1    |
|     12   |  10  |  19  | MOSI   |
|     13   |   9  |  21  | MISO   |
|     14   |  11  |  23  | SCLK   |
|     15   |  14  |   8  | TxD    | 
|     16   |  15  |  10  | RxD    |
|     17   |  28  |   3  | GPIO 8 |
|     18   |  29  |   4  | GPIO 9 |
|     19   |  30  |   5  | GPIO10 |
|     20   |  31  |   6  | GPIO11 |
+----------+------+------+--------+

END OF COMMENTS
 */

//* see Note A in RPi README
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <wiringPi.h>

//* see Note B in RPi README
//
//-------------------------------------------------------------------------
//Special fast timer stuff
#define BCM2708_PERI_BASE       0x20000000
#define SYST_BASE               (BCM2708_PERI_BASE + 0x3000)    /* SYST controller */
#define SYST_CS                 0x0
#define SYST_CLO                0x4
#define SYST_CHI                0x8

#include <dirent.h>
#include <fcntl.h>
#include <assert.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <sys/time.h>


#define PAGE_SIZE (4*1024)
#define BLOCK_SIZE (4*1024)

int mem_fd;
char *syst_mem, *syst_map;
volatile unsigned *syst;

void setup_io();
void restore_io();

//-------------------------------------------------------------------------

//* see Note C in RPi README
// Use GPIO Pin 17, which is Pin 0 for wiringPi library
#define IR_MOD_PIN 1	//modulated signal

#define modPULSES 255 //increase until memory is used up, max 256

//* see Note D in RPi README
volatile unsigned int modIR[modPULSES]; //temp store for modulation pulse durations - changed in interrupt
unsigned int j; //used on for loops
unsigned int sum;//used to calculate modulation 
unsigned int sigLen;//records no of samples in modulation calculation 
unsigned int countM2;//used to calculate modulation 
volatile unsigned int countM;//used to calculate modulation pulses
volatile unsigned int flag =0;//used to signal state of modulation measurement

//* see Note E in RPi README
// -------------------------------------------------------------------------
// modTestISR:  called every time state changes on IR learning receiver (modulated signal)
void modTestISR(void) {//in & out of this ISR ASAP (NB: gettimeofday is too slow for this ISR)
	if (flag) return;//ignore if last event not yet handled by main loop
	//once entered keeps going until we get 256 samples
	countM=modPULSES ;//reset and start again

	while(1){//keep going until we get enough samples set flag to true
		while (digitalRead(IR_MOD_PIN));//wait here until we go LOW..first time it should just fall through
		//now we should be LOW  so log time
		modIR[countM--] =*(syst + (SYST_CLO >> 2));
		if (!countM ) break; //we have got what we want so exit
		while (!digitalRead(IR_MOD_PIN));//wait here until we go HIGH
	}
	flag=1;//tell main loop to handle this event
}


//* see Note F in RPi README
//-------------------------------------------------------------------------
void  reportPeriod(){//report period of modulation frequency in nano seconds for more accuracy
  sum=0;// 
  sigLen=0; //
  countM2=0; //

  for (j=1;j<(modPULSES-1);j++){ //i is byte
	sigLen=(modIR[j-1]-modIR[j]); //siglen

   if (sigLen>40 || sigLen<1) {
	modIR[j-1]=0;//finished with it so clear for next time
	continue;//this is the period range length exclude extraneous ones
    }
    sum +=sigLen;// sum is UL
    countM2++; //countM2 is byte
    modIR[j-1]=0;//finished with it so clear for next time
  }
  modIR[j-1]=0;//now clear last one, which was missed in loop

  if (countM2==0) return;//avoid div by zero = nothing to report
  sum =  sum*1000/countM2; //get it in nano secs
	printf( "\nPeriod: %d : %d\n",sum,countM2);
	printf( "Frequency: %d\n",1000000000/sum);
	fflush(stdout); //make sure output is not buffered
	return;
}

//* see Note G in RPi README
//-------------------------------------------------------------------------
void setup_io()
{

        if ((mem_fd = open("/dev/mem", O_RDWR|O_SYNC) ) < 0) {
                printf("can't open /dev/mem \n");
                exit (-1);
        }
        if ((syst_mem = malloc(BLOCK_SIZE + (PAGE_SIZE-1))) == NULL) {
                printf("allocation error \n");
                exit (-1);
        }
        if ((unsigned long)syst_mem % PAGE_SIZE)
                syst_mem += PAGE_SIZE - ((unsigned long)syst_mem % PAGE_SIZE);
        syst_map = (unsigned char *)mmap(
                (caddr_t)syst_mem, BLOCK_SIZE, PROT_READ|PROT_WRITE,
                MAP_SHARED|MAP_FIXED, mem_fd, SYST_BASE);
        if ((long)syst_map < 0) {
                printf("mmap error %d\n", (int)syst_map);
                exit (-1);
        }
        syst = (volatile unsigned *)syst_map;
}

//* see Note H in RPi README
//-------------------------------------------------------------------------
void restore_io()
{
        munmap(syst_map,BLOCK_SIZE);
}

//* see Note I in RPi README
// -------------------------------------------------------------------------
// main
int main(void) {
 //* see Note J in RPi README
 // sets up the wiringPi library
  if (wiringPiSetup () < 0) {
      fprintf (stderr, "Unable to setup wiringPi: %s\n", strerror (errno));
      return 1;
  }

   //* see Note K in RPi README
	pinMode (IR_MOD_PIN, INPUT) ;
	piHiPri (99) ;//1->99, correct value is system dependent

	pullUpDnControl (IR_MOD_PIN, PUD_UP) ;

	setup_io();//set up the fast uSec timer
	

flag=0;

 //* see Note L in RPi README
  if ( wiringPiISR (IR_MOD_PIN, INT_EDGE_FALLING, &modTestISR) < 0 ) {
      fprintf (stderr, "Unable to setup irMOD ISR: %s\n", strerror (errno));
	  restore_io();
      return 1;
  }
 //* see Note M in RPi README
  while ( 1 ) {//now continue with this loop forever, until exit is forced by CTRIL+C on console
		delayMicroseconds(500000);//suspend to wait until measurement received & allow linux to do stuff
		if (flag){
		reportPeriod();
		flag=0;
	}

 }
 
  //* see Note N in RPi README
restore_io();
return 0;
 
}